/**
 * ETH Zrich; Leitprogramm; Binre Suchbume ----- 
 * Das ist die Klasse eines einzelnen Knotens des Binren Suchbaums. 
 * @author Bjrn Steffen, Timur Erdag, Christina Class
 * @version 1.0 
 */
public class AVLNode {
    private int key;
    private AVLNode left, right;
    
    public AVLNode( int key ) {
        this.key = key;
        left=null;
        right=null;
    }
    
    public AVLNode( int key, AVLNode left, AVLNode right ) {
        this.key = key;
        this.left = left;
        this.right = right;
    }
    
    /**
     * getter Methode fr den Schlssel
     * @return den Schlsselwert
     */
    public int getKey()
    {
        return key;
    }
    
    /**
     * setter Methode fr den Schlssel
     * @param value der neue Schlsselwert
     */
    public void setKey(int value)
    {
        key = value;
    }
    
    
    /**
     * getter Methode fr das rechte Kind
     * @return rechtes Kind
     */
    public AVLNode getRight()
    {
        return right;
    }
    
    /**
     * setter Methode fr das rechte Kind
     * @param newRight Referenz auf neues rechtes Kind
     */
    public void setRight(AVLNode newRight)
    {
        right=newRight;
    }
    
    /**
     * getter Methode fr das linke Kind
     * @return linkes Kind
     */
    public AVLNode getLeft()
    {
        return left;
    }
    
    /**
     * setter Methode fr das linke Kind
     * @param newLeft Referenz auf neues linkes Kind
     */
    public void setLeft(AVLNode newLeft)
    {
        left=newLeft;
    }
    
    
    // bestimmt die Hhe eines Teilbaums, rekursiv
    // beim ersten Aufruf wird 1 als depth (Tiefe) bergeben
    private int getHeight(int depth)
    {
        if (this == null) {
            return depth;
        }
        else {
            int leftHeight = depth;
            if (left != null) 
                leftHeight = left.getHeight(depth+1);
            int rightHeight = depth;
            if (right != null) 
                rightHeight = right.getHeight(depth+1);
            if (leftHeight > rightHeight) {
                return leftHeight;
            } else{
                return rightHeight;
            }
        }
    }
    
    /**
     * bestimmt den Balancefaktor eines Knotens
     * @param node Knoten fr den der Balancefaktor bestimmt werden soll
     * @return Wert des Balancefaktors; 0, falls node null ist
     */
    public int getBalanceFactor()
    {
        if(this == null)
            return 0;
        int balance, lheight = 0, rheight = 0;
        if (left != null) 
            lheight = left.getHeight(1);
        if (right != null) 
            rheight = right.getHeight(1);
        balance = rheight - lheight;
        return balance; 
    }
    
    /**
     * Inorder Ausgabe des Wertes und der Balancefaktoren fr den AVL Baum
     * @param node Wurzelknoten des AVL Baumes, der mit inorder ausgegeben werden soll
     */  
    public void inorderWithBalance(AVLNode node) {
        if (node != null) {
            inorderWithBalance(node.getLeft());
            System.out.println ("value: " + node.getKey() + ";  Balancefaktor: " + node.getBalanceFactor());
            inorderWithBalance(node.getRight());
        }
    }
    
    /**
     * rightrotation; diese Methode ist von Ihnen zu implementieren
     * @param node Wurzelknoten des AVL Baumes, der rotiert werden soll
     * @return neue Wurzel
     */  
    public AVLNode rightrotation(AVLNode node)
    {
        return node;
    }
    
    /**
     * leftrotation; diese Methode ist von Ihnen zu implementieren
     * @param node Wurzelknoten des AVL Baumes, der rotiert werden soll
     * @return neue Wurzel
     */
    public AVLNode leftrotation(AVLNode node)
    { 
        return node;
    }
  
}

